#!/usr/bin/python -tt
# -*- coding: utf-8 -*-
#
# Copyright (c) 2020-2021, FICS, University of Florida
# Jungmin Park
# Date : 10/27/2021
# All rights reserved.
#
# ------------------------------------------------------------

import pyvisa as visa
import AES128 as aes
import time
import matplotlib.pyplot as plt
import numpy as np
from random import seed
from random import randint
import os
import sys
import chipwhisperer as cw

def instrument_setup(dutAddr, start, end):
	try:
		rm = visa.ResourceManager()
		scope = rm.open_resource(dutAddr)     # Change to exact instrument ID
		print(scope.query("*IDN?"))
	except Exception as e:
		print("Error creating instance: {0}".format(e))
		sys.exit()
	# Horizontal Setup
	scope.write('HORizontal:DELay:MODe OFF')
	scope.write('HORizontal:POSistion 10')
	scope.write('HORizontal:SCAle 400e-9')
	scope.write('HORizontal:RECOrdlength 10e3')
	# Trigger Setup
	scope.write('TRIGer:A:EDGE:COUPling DC')
	scope.write('TRIGer:A:EDGE:SLOpe rise')
	scope.write('TRIGer:A:EDGE:SOUrce CH2')
	scope.write('TRIGer:A:LEVel:CH2 1.5')
	scope.write('TRIGger:A:HOLDoff:TIMe 1.00E-03')
	#Vertical Setup
	scope.write('CH1:BANDWIDTH 250E6')
	scope.write('CH2:BANDWIDTH 250E6')
	scope.write('CH1:COUPling AC')
	scope.write('CH2:COUPling DC')
	scope.write('CH1:INVert OFF')
	scope.write('CH2:INVert OFF')
	scope.write('CH2:SCALE 1')
	scope.write('CH1:SCALE 5E-03')
	scope.write(':DATa:SOUrce CH1')
	scope.write(":DATa:START " + start)
	scope.write(":DATa:STOP " + end)
	scope.write(':DATa:ENCdg ASCIi')
	scope.write(':DATa:WIDth 1')
	nr_pt = int(scope.query(':WFMOutpre:NR_Pt?'))
	#yunit = scope.query(':WFMOutpre:YUNit?')
	yoff = float(scope.query(':WFMOutpre:YOFF?'))
	yzero = float(scope.query(':WFMOutpre:YZERO?'))
	ymult = float(scope.query(':WFMOutpre:YMUlt?'))
	xincr = float(scope.query(':WFMOutpre:XINcr?'))
	xdely = float(scope.query(':HORizontal:POSition?'))
	print('----- Instrument Connected! -----')
	print('Number of Points: %d' % nr_pt)
	#print('Y unit: %s' % yunit)
	print('Y zero: %f' % yzero)
	print('Y off: %f' % yoff)
	print('Y mult: %f' % ymult)
	print('X incr: %f' % xincr)
	print('X delay: %f' % xdely)
	scope.timeout = 10000
	scope.chunk_size = 20480
	#scope.write('*CLS')
	scope.write('TRIGger:A:HOLDoff:TIMe?')
	print(scope.read())
	return scope, yoff, ymult, yzero, xincr, nr_pt

# Connect to the instrument
dutAddr = "USB::0x0699::0x0408::C021613::INSTR"
scope, yoff, ymult, yzero, xincr, nr_pt = instrument_setup(dutAddr, '1001', '4000')
key = [0x3B, 0xD9, 0x9B, 0x8E, 0x8A, 0x21, 0x76, 0xB8, 0xFA, 0xA8, 0x16, 0x43, 0x9B, 0x5B, 0xAB, 0x84]   # [MSB  ... LSB]
key_i = key[::-1]
print("Key : ", end='')
for i in range(16):
	print("%02x " % key[i], end='')
print('')

    
#N_traces = int(sys.argv[1])
N_traces = 500
plt_on = 1

print("---- Collecting Power Traces ----")
j = 0
trace = []
pt = []
ct_i = []
pt_i = np.zeros(16, dtype=int)
ct = np.zeros(16, dtype=int)
seed()


start = time.time()
while j < (N_traces):
	pt =  [randint(0,255), randint(0,255), randint(0,255), randint(0,255), randint(0,255), randint(0,255), randint(0,255), randint(0,255), 
		   randint(0,255), randint(0,255), randint(0,255), randint(0,255), randint(0,255), randint(0,255), randint(0,255), randint(0,255)]
	pt_i = pt[::-1]
	print("Plaintext : ", end='') 

    
	for i in range(16):
		print("%02x " % pt[i], end='')

	print('')

	ct_i = aes.AES128().enc_128(key_i, pt_i)
	ct = ct_i[::-1]
    
	print("Ciphertext : ", end='')
	for i in range(16):
		print("%02x " % ct[i], end='')

	print('')

	try:
		values = np.array(scope.query_ascii_values('CURVe?'))
	except:
		pass
	scope.clear()

	if j == 0: 
		traces = values
		pts = pt
		cts = ct
	else:
		traces = np.concatenate((traces, values))
		pts = np.concatenate((pts, pt))
		cts = np.concatenate((cts, ct))

	if (plt_on):
		Volts = (values - yoff) * ymult + yzero
		#Time = np.arange(0, xincr * len(Volts), xincr)
		plt.plot(range(len(Volts)), Volts)
		plt.pause(0.01)
		if j != N_traces-1:
			plt.clf()	

	j += 1
	print("Trace %d" % j)

tr_arry = traces.reshape(N_traces, nr_pt)
pt_arry = pts.reshape(N_traces, 16)
ct_arry = cts.reshape(N_traces, 16)
pt_arry = np.array(pt_arry, dtype = np.uint8)
ct_arry = np.array(ct_arry, dtype = np.uint8)

isExist = os.path.exists('./Data')

if not isExist:
	os.makedirs('./Data')

end = time.time()
print("Elasped time: %d seconds" % (end - start))
print("Done!")
np.savetxt('./Data/EM/traces.csv', tr_arry, delimiter = ',')
np.savetxt('./Data/EM/pts.csv', pt_arry, delimiter = ',')
np.savetxt('./Data/EM/cts.csv', ct_arry, delimiter = ',')

# Create a project file and Store data
key = np.array(key)
if os.path.exists('aes128_EM2.cwp'):
	os.remove('aes128_EM.cwp')
	
proj = cw.create_project("aes128_EM2")
for i in range(N_traces):
	trace = cw.Trace(tr_arry[i,:], pt_arry[i,:], ct_arry[i,:], key)
	proj.traces.append(trace)
proj.save()
scope.close()

	

	


